% Chapter_1_bvp4c_eigenproblem.m
%
% An Introduction to Partial Differential Equations with MATLAB
% Third Edition
% Chapter 1: Introduction
% Section 1.9: Eigenvalue Problems
% Computing for Table 1.1 and plotting Figure 1.3 for Example 3
%
%  2024 by Matthew P. Coleman and Vladislav Bukshtynov
% VladislavBukshtynov@yahoo.com
%

close all; clear; clc;

% problem:
% y'' + lambda y = 0, y(0) = y(1) + y'(1) = 0
% normalization condition: y'(0) = 1

% plotting solutions (first five eigenfunctions) obtained by bvp4c
figure(1); hold on;
x = 0:0.01:1;                            % x-interval
lType = {'r','b','--r','--b',':r'};      % line types (different for all cases)
for n = 1:5
  lambda(n) = ((2*n-1)^2)*pi^2/4;        % initial guess for nth eigenvalue
  solinit = bvpinit(x,@guess,lambda(n)); % intial guess for solution
  sol = bvp4c(@odes,@bcs,solinit);       % solve using bvp4c
  lambda(n) = sol.parameters;            % updated nth eigenvalue
  fprintf('n = %u, lambda = %3.3f\n',n,lambda(n)); % table output
  plot(sol.x,sol.y(1,:),lType{n},'LineWidth',2.5); % plotting, just y1 = y(x)
end
xlabel('$x$','Interpreter','latex');
ylabel('$y_n(x)$','Interpreter','latex');
leg = legend(['$n = 1: \lambda = ' num2str(lambda(1)) '$'], ...
             ['$n = 2: \lambda = ' num2str(lambda(2)) '$'], ...
             ['$n = 3: \lambda = ' num2str(lambda(3)) '$'], ...
             ['$n = 4: \lambda = ' num2str(lambda(4)) '$'], ...
             ['$n = 5: \lambda = ' num2str(lambda(5)) '$']);
set(leg,'Interpreter','latex','Location','SouthWest');
set(gca,'FontSize',14,'yLim',[-0.5 0.5],'yTick',-0.4:0.2:0.4);
legend boxoff; box off;

return

% ODE-2 as a system of two ODE-1
function dydx = odes(x,y,lambda)
  dydx = [y(2)          % u = y'
         -lambda*y(1)]; % u' = y'' = -lambda y
end

% boundary conditions
function res = bcs(yl,yr,lambda)
  res = [yl(1)       % y(0)  = 0 (left)
         yr(1)+yr(2) % y(1) + y'(1) = 0 (right)
         yl(2)-1];   % y'(0) = 1 (left, normalization)
end

% initial guess (specific to problem)
function g = guess(x)
  g = [sin(5*pi*x)        % y(x)
       5*pi*cos(5*pi*x)]; % y'(x)
end
