% Chapter_10_gamma_bessel_legendre.m
%
% An Introduction to Partial Differential Equations with MATLAB
% Third Edition
% Chapter 10: PDEs in Higher Dimensions
% Section 10.4: Interlude 1: Bessel Functions
%
%  2024 by Matthew P. Coleman and Vladislav Bukshtynov
% VladislavBukshtynov@yahoo.com
%

close all; clear; clc;
set(groot, 'DefaultFigureColormap', jet);

% Plotting Figure 10.9 for Section 10.4
% visualizing gamma using built-in MATLAB function gamma(x)
x = -5:0.001:5;
figure(1); hold on;
plot(x, gamma(x),'r-','lineWidth',2);
axis([-5 5 -30 30]); 
plot([-5 5],[0 0],'k--','lineWidth',1);
plot([0 0],[-30 30],'k--','lineWidth',1);
xlabel('x'); ylabel('\Gamma(x)');
set(gca,'FontSize',14,'xTick',-5:1:5); box off;

% Plotting Figure 10.10(a) for Section 10.4
% Bessel functions of the first kind
xx = 0:0.001:20;                              % discretizing x-interval
figure(2); hold on;
plot(xx,besselj(0,xx),'r-','LineWidth',2);    % alpha = 0
plot(xx,besselj(1,xx),'r--','LineWidth',2);   % alpha = 1
plot(xx,besselj(2,xx),'b-','LineWidth',2);    % alpha = 2
plot(xx,besselj(3,xx),'b--','LineWidth',2);   % alpha = 3
plot([0 20],[0 0],'k--','lineWidth',1);
xlabel('x'); ylabel('J_\alpha(x)');
title('Bessel functions of the first kind: J_\alpha(x)');
legend('\alpha = 0','\alpha = 1','\alpha = 2','\alpha = 3','Location','NorthEast');
set(gca,'FontSize',14); box off; legend boxoff;

% Plotting Figure 10.10(b) for Section 10.4
% Bessel functions of the second kind (Weber functions)
figure(3); hold on;
plot(xx,bessely(0,xx),'r-','LineWidth',2);    % alpha = 0
plot(xx,bessely(1,xx),'r--','LineWidth',2);   % alpha = 1
plot(xx,bessely(2,xx),'b-','LineWidth',2);    % alpha = 2
plot(xx,bessely(3,xx),'b--','LineWidth',2);   % alpha = 3
plot([0 20],[0 0],'k--','lineWidth',1);
xlabel('x'); ylabel('Y_\alpha(x)');
title('Bessel functions of the second kind: Y_\alpha(x)');
legend('\alpha = 0','\alpha = 1','\alpha = 2','\alpha = 3','Location','NorthEast');
set(gca,'FontSize',14,'yLim',[-1 1]); box off; legend boxoff;

% Plotting Figure 10.11(a) for Section 10.4
% modified Bessel functions of the first kind
xx = 0:0.001:5;                               % discretizing x-interval
figure(4); hold on;
plot(xx,besseli(0,xx),'r-','LineWidth',2);    % alpha = 0
plot(xx,besseli(1,xx),'r--','LineWidth',2);   % alpha = 1
plot(xx,besseli(2,xx),'b-','LineWidth',2);    % alpha = 2
plot(xx,besseli(3,xx),'b--','LineWidth',2);   % alpha = 3
xlabel('x'); ylabel('I_\alpha(x)');
title('modified Bessel functions of the first kind: I_\alpha(x)');
legend('\alpha = 0','\alpha = 1','\alpha = 2','\alpha = 3','Location','NorthWest');
set(gca,'FontSize',14,'yLim',[0 30]); box off; legend boxoff;

% Plotting Figure 10.11(b) for Section 10.4
% modified Bessel functions of the second kind (Weber functions)
xx = 0:0.001:10;                              % discretizing x-interval
figure(5); hold on;
plot(xx,besselk(0,xx),'r-','LineWidth',2);    % alpha = 0
plot(xx,besselk(1,xx),'r--','LineWidth',2);   % alpha = 1
plot(xx,besselk(2,xx),'b-','LineWidth',2);    % alpha = 2
plot(xx,besselk(3,xx),'b--','LineWidth',2);   % alpha = 3
xlabel('x'); ylabel('K_\alpha(x)');
title('modified Bessel functions of the second kind: K_\alpha(x)');
legend('\alpha = 0','\alpha = 1','\alpha = 2','\alpha = 3','Location','NorthEast');
set(gca,'FontSize',14,'yLim',[0 1]); box off; legend boxoff;

% Plotting Figure 10.12 for Section 10.5
% Legendre polynomials
xx = -1:0.001:1;                              % discretizing x-interval
figure(6); hold on;
plot(xx,legendreP(0,xx),'k-','LineWidth',2);  % n = 0
plot(xx,legendreP(1,xx),'k--','LineWidth',2); % n = 1
plot(xx,legendreP(2,xx),'b-','LineWidth',2);  % n = 2
plot(xx,legendreP(3,xx),'b--','LineWidth',2); % n = 3
plot(xx,legendreP(4,xx),'r-','LineWidth',2);  % n = 4
plot(xx,legendreP(5,xx),'r--','LineWidth',2); % n = 5
xlabel('x'); ylabel('P_n(x)');
title('Legendre polynomials: P_n(x)');
legend('P_0','P_1','P_2','P_3','P_4','P_5','Location','South','Orientation','Horizontal');
set(gca,'FontSize',14,'yLim',[-1.1 1.1]); box off; legend boxoff;
