% Chapter_11_finite_element_method.m
%
% An Introduction to Partial Differential Equations with MATLAB
% Third Edition
% Chapter 11: Overview of Spectral, Finite Elements, and Finite Volumes Methods
% Section 11.2: Finite Element Methods
% Solving the Heat Equation by finite element Galerkin approximation
%
%  2024 by Matthew P. Coleman and Vladislav Bukshtynov
% VladislavBukshtynov@yahoo.com
%

close all; clear; clc;
set(groot, 'DefaultFigureColormap', jet);

% Example 3 (used in Sections 7.1 & 11.1):
% u_t = u_xx, 0 < x < 1, t > 0
% IC:  u(x,0) = sin(pi*x)
% BCs: u(0,t) = u(1,t) = 0

% defining constants
x0 = 0; xf = 1;   % space interval
t0 = 0; tf = 0.2; % time interval
n = 6; m = 11;    % number of points (space & time)

% discretization
dx = (xf-x0)/(n-1);        % step size for space
dt = (tf-t0)/(m-1);        % step size for time
xd1 = x0:dx:xf;            % x-discretization
td1 = t0:dt:tf;            % t-discretization
[X,T] = meshgrid(xd1,td1); % creating mesh grid

% analytical solution
figure(1);
uAn = exp(-pi^2*T).*sin(pi*X);
surf(X,T,uAn);
xlabel('x'); ylabel('t'); zlabel('u(x,t)');
title('Analytical Solution');
set(gca,'FontSize',14); view([-32 12]); box off;

% solution by finite element Galerkin approximation
U1 = heatEqn_fem(n,m,xd1,dx,dt);
figure(2);
surf(X,T,U1);
xlabel('x'); ylabel('t'); zlabel('u(x,t)');
title('Solution by Finite Element Galerkin Approximation');
set(gca,'FontSize',14); view([-32 12]); box off;

% error analysis for fixed x-values
relErr1 = 100*abs(uAn-U1)./abs(uAn);

% solution with finer x-discretization
n = 11;                    % changing number of points (space only)
dx = (xf-x0)/(n-1);        % updating step size for space
xd2 = x0:dx:xf;            % new x-discretization
[X,T] = meshgrid(xd2,td1); % creating mesh grid
uAn = exp(-pi^2*T).*sin(pi*X); % analytical solution
U2 = heatEqn_fem(n,m,xd2,dx,dt);
figure(3); surf(X,T,U2);
xlabel('x'); ylabel('t'); zlabel('u(x,t)');
title('Finite Element Galerkin Approximation (finer x)');
set(gca,'FontSize',14); view([-32 12]); box off;

% error analysis for fixed x-values
relErr2 = 100*abs(uAn-U2)./abs(uAn);

% solution with finer t-discretization
n = 6; m = 21;             % number of points (space & time)
dx = (xf-x0)/(n-1);        % step size for space
dt = (tf-t0)/(m-1);        % step size for time
xd1 = x0:dx:xf;            % x-discretization
td2 = t0:dt:tf;            % t-discretization
[X,T] = meshgrid(xd1,td2); % creating mesh grid
uAn = exp(-pi^2*T).*sin(pi*X); % analytical solution
U3 = heatEqn_fem(n,m,xd1,dx,dt);
figure(4); surf(X,T,U3);
xlabel('x'); ylabel('t'); zlabel('u(x,t)');
title('Finite Element Galerkin Approximation (finer t)');
set(gca,'FontSize',14); view([-32 12]); box off;

% error analysis for fixed x-values
relErr3 = 100*abs(uAn-U3)./abs(uAn);

% solution with finer x- and t-discretization
n = 11;                    % number of points (space)
dx = (xf-x0)/(n-1);        % updating step size for space
xd2 = x0:dx:xf;            % new x-discretization
[X,T] = meshgrid(xd2,td2); % creating mesh grid
uAn = exp(-pi^2*T).*sin(pi*X); % analytical solution
U4 = heatEqn_fem(n,m,xd2,dx,dt);
figure(5); surf(X,T,U4);
xlabel('x'); ylabel('t'); zlabel('u(x,t)');
title('Finite Element Galerkin Approximation (finer x and t)');
set(gca,'FontSize',14); view([-32 12]); box off;

% error analysis for fixed x-values
relErr4 = 100*abs(uAn-U4)./abs(uAn);

% plotting errors
figure(6); hold on;
plot(td1,relErr1(:,2),'.k','MarkerSize',10);
plot(td1,relErr1(:,3),'ok','MarkerSize',8);
plot(td1,relErr1(:,4),'ok','MarkerSize',12);
plot(td1,relErr2(:,2),'.r','MarkerSize',10);
plot(td1,relErr2(:,3),'or','MarkerSize',8);
plot(td1,relErr2(:,4),'or','MarkerSize',12);
plot(td2,relErr3(:,2),'.b','MarkerSize',10);
plot(td2,relErr3(:,3),'ob','MarkerSize',8);
plot(td2,relErr3(:,4),'ob','MarkerSize',12);
plot(td2,relErr4(:,2),'.m','MarkerSize',10);
plot(td2,relErr4(:,3),'om','MarkerSize',8);
plot(td2,relErr4(:,4),'om','MarkerSize',12);
xlabel('t'); ylabel('error, %');
legend('x = 0.2', 'x = 0.4', 'x = 0.6',...
    'x = 0.2 (finer x)', 'x = 0.4', 'x = 0.6',...
    'x = 0.2 (finer t)', 'x = 0.4', 'x = 0.6',...
    'x = 0.2 (finer x & t)', 'x = 0.4', 'x = 0.6','Location','NorthWest');
set(gca,'FontSize',14); legend boxoff; box off;

return

% finite element Galerkin approximation
function sol = heatEqn_fem(n,m,xd,dx,dt)

  % defining tent/hat (symbolic) functions
  syms v(x)
  for i = 2:n-1
    v(x) = piecewise((x>=xd(i-1))&(x<=xd(i)),(x-xd(i-1))/dx,...
        (x>=xd(i))&(x<=xd(i+1)),1-(x-xd(i))/dx,0);
    V{i} = v(x);
  end
  
  % computing matrix A
  A = zeros(n-2,n-2);
  for i = 2:n-1
    for j = 2:n-1
      A(i-1,j-1) = int(diff(V{i})*diff(V{j})+V{i}*V{j}/dt,xd(1),xd(end));
    end
  end

  % solution matrix with added initial condition
  U = zeros(n,m);
  U(:,1) = sin(pi*xd');
  
  % marching in time to solve BVP
  for k = 2:m
 
    % RHS vector b
    b = zeros(n-2,1);
    for j = 2:n-1
      b(j-1) = trapz(xd,U(:,k-1).*subs(V{j},xd')/dt);
    end
    
    % solving system of linear equations for coefficients c
    c = A^-1 * b;
  
    % evaluating FE approximation v(x)
    v = 0;
    for i = 2:n-1
      v = v + c(i-1)*V{i};
    end
    U(:,k) = subs(v,xd);  % updating solution
  end
  
  sol = U';               % complete solution
end
