% Chapter_11_spectral_method.m
%
% An Introduction to Partial Differential Equations with MATLAB
% Third Edition
% Chapter 11: Overview of Spectral, Finite Elements, and Finite Volumes Methods
% Section 11.1: Spectral Methods
% Solving the Heat Equation by Fourier Sine Spectral Method
%
%  2024 by Matthew P. Coleman and Vladislav Bukshtynov
% VladislavBukshtynov@yahoo.com
%

close all; clear; clc;
set(groot, 'DefaultFigureColormap', jet);

% Example 4 (used in Section 7.1):
% u_t = u_xx, 0 < x < 1, t > 0
% IC:  u(x,0) = sin(pi*x)
% BCs: u(0,t) = u(1,t) = 0

% defining constants
x0 = 0; xf = 1;   % space interval
t0 = 0; tf = 0.2; % time interval
n = 6; m = 11;    % number of points (space & time)
N = 10;           % upper limit for summation

% discretization
dx = (xf-x0)/(n-1);     % step size for space
dt = (tf-t0)/(m-1);     % step size for time
x = x0:dx:xf;           % x-discretization
t1 = t0:dt:tf;          % t-discretization
[X,T] = meshgrid(x,t1); % creating mesh grid

% analytical solution
figure(1);
uAn = exp(-pi^2*T).*sin(pi*X);
surf(X,T,uAn);
xlabel('x'); ylabel('t'); zlabel('u(x,t)');
title('Analytical Solution');
set(gca,'FontSize',14); view([-32 12]); box off;

% solution by Fourier sine spectral method
Bini = [1 zeros(1,N-1)];                % from initial condition: b1(t0) = 1
U1 = heatEqn_spectral(Bini,n,m,N,x,dt); % solution

% error analysis for fixed x-values
relErr1 = 100*abs(uAn-U1)./abs(uAn);

% plotting numerical solution
figure(2);
surf(X,T,U1);
xlabel('x'); ylabel('t'); zlabel('u(x,t)');
title('Solution by Fourier Sine Spectral Method');
set(gca,'FontSize',14); view([-32 12]); box off;

% solution by Fourier sine spectral method with finer t-discretization
m = 21;
dt = (tf-t0)/(m-1);     % new step size for time
t2 = t0:dt:tf;          % new t-discretization
[X,T] = meshgrid(x,t2); % changing mesh grid
uAn = exp(-pi^2*T).*sin(pi*X);          % updated analytical solution
U2 = heatEqn_spectral(Bini,n,m,N,x,dt); % solution by spectral method

% error analysis for fixed x-values
relErr2 = 100*abs(uAn-U2)./abs(uAn);

% plotting errors
figure(3); hold on;
plot(t1,relErr1(:,2),'.b','MarkerSize',10);
plot(t1,relErr1(:,3),'ob','MarkerSize',8);
plot(t1,relErr1(:,4),'ob','MarkerSize',12);
plot(t2,relErr2(:,2),'.r','MarkerSize',10);
plot(t2,relErr2(:,3),'or','MarkerSize',8);
plot(t2,relErr2(:,4),'or','MarkerSize',12);
xlabel('t'); ylabel('error, %');
legend('x = 0.2 (coarser t)', 'x = 0.4', 'x = 0.6',...
    'x = 0.2 (finer t)', 'x = 0.4', 'x = 0.6','Location','NorthWest');
set(gca,'FontSize',14); legend boxoff; box off;

return

% Fourier sine spectral method
function sol = heatEqn_spectral(Bini,n,m,N,x,dt)
  B = zeros(N,m); % matrix of b_k coefficients
  B(:,1) = Bini;  % initial conditions: b_k(t0)
  U = zeros(n,m); % solution matrix
  
  % marching in time: updating b_k(t) coefficients
  for j = 2:m
    for k = 1:N
      B(k,j) = B(k,j-1)/(1 + pi^2 * k^2 * dt);
    end
  end
  
  % marching in time: updating solution marix
  for j = 1:m
    for k = 1:N
      U(:,j) = U(:,j) + B(k,j)*sin(k*pi*x');
    end
  end
  sol = U'; % complete solution
end
