% Chapter_3_BVPs_bvp4c.m
%
% An Introduction to Partial Differential Equations with MATLAB
% Third Edition
% Chapter 3: Using MATLAB for Solving DEs and Visualizing Solutions
% Section 3.3: Solving BVPs Numerically Using bvp4(5)c
%
%  2024 by Matthew P. Coleman and Vladislav Bukshtynov
% VladislavBukshtynov@yahoo.com
%

close all; clear; clc;

% problem:
% y'' + lambda y = 0, y(0) = y(1) = 0
% normalization condition: y'(0) = 1, i.e., Cn = 1/(n*pi)

% plotting solutions (first five eigenfunctions) obtained by bvp4c
figure(1); hold on;                      % figure #1 (solutions)
figure(2); hold on;                      % figure #2 (errors)
x = 0:0.01:1;                            % x-interval
lType = {'r','b','--r','--b',':r'};      % line types (different for all cases)
for n = 1:5
  lambda = (n*pi)^2;                     % initial guess for nth eigenvalue
  solinit = bvpinit(x,@guess,lambda);    % initial guess for solution
  sol = bvp4c(@odes,@bcs,solinit);       % solve using bvp4c
  figure(1);                             % plotting y(1) = y(x)
  plot(sol.x,sol.y(1,:),lType{n},'LineWidth',2.5);
  solEx = (1/(n*pi))*sin(n*pi*sol.x);    % computing exact y(x)
  figure(2);                             % creating error plot
  plot(sol.x,abs(sol.y(1,:)-solEx),lType{n},'LineWidth',2.5);
end

% figure #1 settings
figure(1);
xlabel('x'); ylabel('y_n(x)');
legend('n = 1','n = 2','n = 3','n = 4','n = 5','Location','SouthWest');
set(gca,'FontSize',14,'yLim',[-0.3 0.4]); legend boxoff; box off;

% figure #2 settings
figure(2);
xlabel('x'); ylabel('error, \epsilon_n(x)');
legend('n = 1','n = 2','n = 3','n = 4','n = 5','Location','NorthWest');
set(gca,'FontSize',14); legend boxoff; box off;

return

% ODE-2 as a system of two ODE-1
function dydx = odes(x,y,lambda)
  dydx = [y(2)          % u = y'
         -lambda*y(1)]; % u' = y'' = -lambda y
end

% boundary conditions
function res = bcs(yl,yr,lambda)
  res = [yl(1)          % y(0)  = 0 (left)
         yr(1)          % y(1)  = 0 (right)
         yl(2)-1];      % y'(0) = 1 (left, normalization)
end

% initial guess (specific to problem)
function g = guess(x)
  g = [sin(x)           % y(x)
       cos(x)];         % y'(x)
end
