% Chapter_3_PDEs_pdepe.m
%
% An Introduction to Partial Differential Equations with MATLAB
% Third Edition
% Chapter 3: Using MATLAB for Solving DEs and Visualizing Solutions
% Section 3.4: Solving PDEs Numerically by pdepe
%
%  2024 by Matthew P. Coleman and Vladislav Bukshtynov
% VladislavBukshtynov@yahoo.com
%

close all; clear; clc;
set(groot, 'DefaultFigureColormap', jet);

% Exercise 3, Section 2.6
% heat equation: u_t = u_xx
% BCs: u_x(0,t) = u(pi,t) = 0
% IC:  u(x,0) = 7cos(5x/2)
% analytical solution: u(x,t) = 7*e^(-25t/4)*cos(5x/2)

% visualizing analytical solution

% using surface and mesh plots
figure(1);
x = linspace(0,pi,30);            % discretizing x-interval
t = linspace(0,1,10);             % discretizing t-interval
[X,T] = meshgrid(x,t);            % creating (x,t)-grid
uAn = 7*exp(-25*T/4).*cos(5*X/2); % solution function on (x,t)-grid
surf(X,T,uAn);                    % surface plot using surf
xlabel('distance, x'); ylabel('time, t'); zlabel('temperature, u(x,t)');
set(gca,'FontSize',14,'zLim',[-8 8]); box off; view([-28 15]);
figure(2);
mesh(X,T,uAn);                    % mesh plot using mesh
xlabel('distance, x'); ylabel('time, t'); zlabel('temperature, u(x,t)');
set(gca,'FontSize',14,'zLim',[-8 8]); box off; view([-28 15]);

% 2D cross-section plots u(t) for fixed x
figure(3); hold on;
lType = {'r','b','--r','--b',':r',':b'}; % line types (different for all cases)
xx = [1 10 20 30];
for n = 1:length(xx)
  plot(T(:,xx(n)),uAn(:,xx(n)),lType{n},'LineWidth',2);
end
xlabel('time, t'); ylabel('temperature, u(t)');
legend('x = 0', 'x = 0.9750', 'x = 2.0583', 'x = 3.1416');
set(gca,'FontSize',14); legend boxoff; box off;

% 2D cross-section plots u(x) for fixed t
figure(4); hold on;
tt = [1 2 4 6 10];
for n = 1:length(tt)
  plot(X(tt(n),:),uAn(tt(n),:),lType{n},'LineWidth',2);
end
xlabel('distance, x'); ylabel('temperature, u(x)');
legend('t = 0', 't = 0.1111', 't = 0.3333', 't = 0.5556', 't = 1.0',...
    'Location','Best');
set(gca,'FontSize',14); legend boxoff; box off;

% 2D contour plot
figure(5);
contour(X,T,uAn,-7:1:7,'ShowText','on','LineWidth',2);
xlabel('distance, x'); ylabel('time, t');
set(gca,'FontSize',14); box off;

% 2D image with scaled colors
figure(6);
imagesc(linspace(0,pi,30),linspace(0,1,10),uAn);
colorbar; xlabel('distance, x'); ylabel('time, t');
set(gca,'FontSize',14); box off;

% Solving heat (parabolic) equation using pdepe
% 
% PDE must be written in the form:
%     c(x,t,u,u_x)*u_t = x^(-m)*d/dx(x^m *f(x,t,u,u_x)) + s(x,t,u,u_x)
% for functions c, b, s, and symmetry m
% 
% with BCs: p(xl,t,u) + q(xl,t)*f(xl,t,u,u_x) = 0   (left boundary)
%           p(xr,t,u) + q(xr,t)*f(xr,t,u,u_x) = 0   (right boundary)
% and IC: u(x,0) = g(x)
figure(7);
x = linspace(0,pi,30);              % discretizing x-interval
t = linspace(0,1,10);               % discretizing t-interval
m = 0;                              % 1D case with no symmetry
u = pdepe(m,@heatEqn,@ic,@bcs,x,t); % solving PDE
surf(x,t,u);                        % plotting solution
xlabel('distance, x'); ylabel('time, t'); zlabel('temperature, u(x,t)');
set(gca,'FontSize',14,'zLim',[-8 8]); box off; view([-28 15]);

% error plot to compare analytical & numerical solutions
figure(8);
surf(x,t,abs(u-uAn));
xlabel('distance, x'); ylabel('time, t'); zlabel('error, \epsilon(x,t)');
set(gca,'FontSize',14); box off; view([-28 15]);

% Solving the same PDE with differents initial conditions
u2 = pdepe(m,@heatEqn,@ic2,@bcs,x,t); % with IC #2
u3 = pdepe(m,@heatEqn,@ic3,@bcs,x,t); % with IC #3
figure(9); surf(x,t,u2);              % plotting solution u2
xlabel('distance, x'); ylabel('time, t'); zlabel('temperature, u(x,t)');
set(gca,'FontSize',14,'zLim',[-8 8]); box off; view([-28 15]);
figure(10); surf(x,t,u3);              % plotting solution u3
xlabel('distance, x'); ylabel('time, t'); zlabel('temperature, u(x,t)');
set(gca,'FontSize',14,'zLim',[-1 1]); box off; view([-22 6]);

return;

% specify c, f, s to define PDE: 1*u_t = d/dx(du/dx) + 0
function [c,f,s] = heatEqn(x,t,u,dudx)
  c = 1;
  f = dudx;
  s = 0;
end

% boundary conditions
function [pl,ql,pr,qr] = bcs(x1,ul,xr,ur,t)
  pl = 0;  ql = 1; % left:  0 + 1*dudx = 0, i.e.,  u_x(0,t) = 0
  pr = ur; qr = 0; % right: u + 0 = 0,      i.e.,  u(pi,t)  = 0
end

% initial condition IC #1
function value = ic(x)
  value = 7*cos(5*x/2);
end

% initial condition IC #2
function value = ic2(x)
  value = 7*sin(5*x/2);
end

% initial condition IC #3
function value = ic3(x)
  value = cos(x);
end
