% Chapter_3_symbolic_math_IVPs.m
%
% An Introduction to Partial Differential Equations with MATLAB
% Third Edition
% Chapter 3: Using MATLAB for Solving DEs and Visualizing Solutions
% Section 3.2: Symbolic Math Toolbox for Solving ODEs
% Solving IVPs (3.6)-(3.9)
%
%  2024 by Matthew P. Coleman and Vladislav Bukshtynov
% VladislavBukshtynov@yahoo.com
%

close all; clear; clc;

figure(1); hold on;

% Logistic equation: dP/dt = P(a-bP), P(0) = P0 = 2
disp('(a) logistic equation, solution');
a = 1; b = 0.1; P0 = 2;           % ODE constants
syms P(t);                        % creating symbolic function P(t)
ode = diff(P,t) == P*(a-b*P);     % defining ODE
cond = P(0) == P0;                % setting IC
Psol(t) = dsolve(ode,cond);       % solving IVP using dsolve
Psol = simplify(Psol)             % simplifying solution
tt = 0:0.01:5;                    % time scale discretization (for plotting)
P1 = eval(Psol(tt));              % evaluating the solution over grid tt
plot(tt,P1,'-r','LineWidth',2);   % plotting solution

% Logistic equation with source: dP/dt = P(a-bP) + h, h = 2
disp('(b) logistic equation with source, solution');
h = 2;                            % ODE constant
ode = diff(P,t) == P*(a-b*P) + h; % defining ODE
Psol(t) = dsolve(ode,cond);       % solving IVP using dsolve
Psol = simplify(Psol)             % simplifying solution
P2 = eval(Psol(tt));              % evaluating the solution over grid tt
plot(tt,P2,'--r','LineWidth',2);  % plotting solution

% Logistic equation with immigration: dP/dt = P(a-bP) + c e^(-kP)
% c = 200, k = 0.5
disp('(c) logistic equation with immigration, solution');
c = 200; k = 0.5;                           % ODE constants
ode = diff(P,t) == P*(a-b*P) + c*exp(-k*P); % defining ODE
Psol(t) = dsolve(ode,cond);                 % solving IVP using dsolve
Psol = simplify(Psol)                       % simplifying solution

% Gompertz equation: dP/dt = P(a-b*ln(P))
disp('(d) Gompertz equation, solution');
ode = diff(P,t) == P*(a-b*log(P)); % defining ODE
Psol(t) = dsolve(ode,cond);        % solving IVP using dsolve
Psol = simplify(Psol)              % simplifying solution
P3 = eval(Psol(tt));               % evaluating the solution over grid tt
plot(tt,P3,'-b','LineWidth',2);    % plotting solution

axis([0 5 0 20]); xlabel('t'); ylabel('P(t)');
plot([0 5],[a/b a/b],'--k');      % plotting asymptote P(t) = a/b
plot(0,P0,'.r','MarkerSize',30);  % plotting IC
legend('logistic equation','logistic with source',...
       'Gompertz equation','Location','SouthEast');
set(gca,'FontSize',14); legend boxoff; box off;
