% Chapter_3_visualize_ODEs.m
%
% An Introduction to Partial Differential Equations with MATLAB
% Third Edition
% Chapter 3: Using MATLAB for Solving DEs and Visualizing Solutions
% Section 3.1: Visualizing Solutions for ODEs
% Examples 1-4.
%
%  2024 by Matthew P. Coleman and Vladislav Bukshtynov
% VladislavBukshtynov@yahoo.com
%

close all; clear; clc;

% Example 1: Family of solutions

% xy' - y = x^2 sin(x)
x = -5:0.01:5;     % x-interval
c = [-3 -1 0 1 3]; % choices of constant c
figure(1); hold on;
lType = {'r','b','--r','--b',':r'}; % line types (different for all cases)
for i = 1:length(c)
  y = f_sol_1(x,c(i));
  plot(x,y,lType{i},'LineWidth',2);
end
xlabel('x'); ylabel('y');
legend('c = -3','c = -1','c = 0','c = 1','c = 3','Location','North');
set(gca,'FontSize',14); legend boxoff; box off;
pause;             % pausing output - to proceed push any key

% y'' - 2y' + y = 0
x = -3:0.01:3;     % x-interval
c1 = [0 3 5];      % choices of constant c1
c2 = [5 0 -2];     % choices of constant c2
figure(2); hold on;
for i = 1:length(c1)
  y = f_sol_2(x,c1(i),c2(i));
  plot(x,y,lType{i},'LineWidth',2);
end
xlabel('x'); ylabel('y');
legend('c_1 = 0, c_2 = 5','c_1 = 3, c_2 = 0','c_1 = 5, c_2 = -2','Location','South');
set(gca,'FontSize',14); legend boxoff; box off;
axis([-3 3 -10 10]);
pause;

% Example 2: dynamical plotting of family of solutions
% y'' - y = 0
% y = c1*exp(x) + c2*exp(-x), c1 > c2 (red)
x = -3:0.01:3;     % x-interval
figure(3); hold on;
for c1 = -5:1:5
  for c2 = -5:1:5
    y = f_sol_3(x,c1,c2);
    if c1>c2
      plot(x,y,'r-','LineWidth',1);
    else
      plot(x,y,'b-','LineWidth',1);
    end
    axis([-3 3 -10 10]); pause(0.1);
  end
end
xlabel('x'); ylabel('y');
set(gca,'FontSize',14); box off;
pause;

% y = c1*exp(x) + c2*exp(-x), |c1| > |c2| (red)
figure(4); hold on;
for c1 = -5:1:5
  for c2 = -5:1:5
    y = f_sol_3(x,c1,c2);
    if abs(c1) > abs(c2)
      plot(x,y,'r-','LineWidth',1);
    else
      plot(x,y,'b-','LineWidth',1);
    end
    axis([-3 3 -10 10]); pause(0.1);
  end
end
xlabel('x'); ylabel('y');
set(gca,'FontSize',14); box off;
pause;

% Example 3: direction field
% y' = 0.2xy
x = -5:1:5; y = -5:1:5;  % x- and y-intervals
[xx,yy] = meshgrid(x,y); % creating xy-grid
v = 0.2*xx.*yy;          % direction field vector (u,v)
u = ones(size(v));
figure(5); hold on;
quiver(xx,yy,u,v,'LineWidth',1,'Color','k'); % plotting with quiver
xlabel('x'); ylabel('y');
axis([-6 6 -7 7]); set(gca,'FontSize',14); box off;
pause;

% adding solution curves y = c*exp(0.1*x^2)
x = -6:0.01:6;           % updated x-interval (with smaller step)
for c = [0.5 1 2 4]
  yyy = f_sol_4(x,c);
  plot(x,yyy,'r-','LineWidth',2);
end
for c = [-1 -2 -5]
  yyy = f_sol_4(x,c);
  plot(x,yyy,'b-','LineWidth',2);
end
for c = 0
  yyy = f_sol_4(x,c);
  plot(x,yyy,'k-','LineWidth',2);
end
pause;

% Example 4: solution (implicit): G(x,y) = c
% dy/dx = x(1-x)/(y(y-2)) (separable ODE)
% G(x,y) = y^3/3 - y^2 + x^3/3 - x^2/2 = c
f=@(x,y) y.^3/3 - y.^2 + x.^3/3 - x.^2/2; % defining a function through a function handle
[X,Y] = meshgrid(-2:0.01:3); % creating xy-grid (same for x and y)
Z = f(X,Y);                  % evaluating solution on the meshgrid
figure(6); hold on;
contour(X,Y,Z,-5:1:5,'ShowText','on','LineColor','k','LineWidth',1.5); pause;
contour(X,Y,Z,-0.4:0.1:0.4,'ShowText','on','LineColor','r','LineWidth',1.5); pause;
contour(X,Y,Z,-1.4:0.1:-0.6,'ShowText','on','LineColor','b','LineWidth',1.5);
xlabel('x'); ylabel('y');
set(gca,'FontSize',14); box off;

return

% functions defined manually
function y = f_sol_1(x,c)
  y = c*x - x.*cos(x);
end
function y = f_sol_2(x,c1,c2)
  y = c1*exp(x) + c2*x.*exp(x);
end
function y = f_sol_3(x,c1,c2)
  y = c1*exp(x) + c2*exp(-x);
end
function y = f_sol_4(x,c)
  y = c*exp(0.1*x.^2);
end
