% Chapter_5_heat_sol_visualize.m
%
% An Introduction to Partial Differential Equations with MATLAB
% Third Edition
% Chapter 5: Solving the Big Three PDEs on Finite Domains
% Section 5.1: Solving the Homogeneous Heat Equation for a Finite Rod
% Examples 1-2.
%
%  2024 by Matthew P. Coleman and Vladislav Bukshtynov
% VladislavBukshtynov@yahoo.com
%

close all; clear; clc;
set(groot, 'DefaultFigureColormap', jet);

% Example 1
% heat equation: u_t = 3u_xx
% BCs: u_x(0,t) = u(pi,t) = 0
% IC:  u(x,0) = x*(pi-x)

% visualizing analytical solution using surface plot
figure(1);
x = linspace(0,pi,10);            % discretizing x-interval
t = linspace(0,1.5,10);           % discretizing t-interval
[X,T] = meshgrid(x,t);            % creating (x,t)-grid
uAn = zeros(length(X),length(T)); % solution function on (x,t)-grid
for k = 1:10                      % updating solution via truncated summation
  uAn = uAn + (8/pi)*exp(-3*(2*k-1)^2*T).*sin((2*k-1)*X)/(2*k-1)^3;
end
surf(X,T,uAn);                    % surface plot using surf
xlabel('distance, x'); ylabel('time, t'); zlabel('temperature, u(x,t)');
set(gca,'FontSize',14,'zLim',[0 2.5]); box off; view([-47 15]);

% 2D cross-section plots u(x) for fixed t
figure(2); hold on;
lType = {':r',':b','--r','--b','r','b'}; % line types (different for all cases)
x = linspace(0,pi,100);                  % discretizing (finer) x-interval
t = [0 0.25 0.5 0.75 1 1.5];             % discretizing (new) t-interval
for n = 1:length(t)
  uAn = zeros(size(x));                  % solution function on x-grid
  for k = 1:10                           % updating solution via truncated summation
    uAn = uAn + (8/pi)*exp(-3*(2*k-1)^2*t(n))*sin((2*k-1)*x)/(2*k-1)^3;
  end
  plot(x,uAn,lType{n},'LineWidth',2);
end
xlabel('distance, x'); ylabel('temperature, u(x)');
legend('t = 0','t = 0.25','t = 0.5','t = 0.75','t = 1','t = 1.5','Location','North');
set(gca,'FontSize',14); legend boxoff; box off;
pause;                                   % pausing output - to proceed push any key

% Example 2
% heat equation: u_t = u_xx
% BCs: u_x(0,t) = u_x(3,t) = 0
% IC:  u(x,0) = x

% visualizing analytical solution using surface plot
figure(3);
x = linspace(0,3,10);              % discretizing x-interval
t = linspace(0,4,10);              % discretizing t-interval
[X,T] = meshgrid(x,t);             % creating (x,t)-grid
uAn = 3*ones(length(X),length(T)); % solution function on (x,t)-grid
for k = 1:50                       % updating solution via truncated summation
  uAn = uAn - (12/pi^2)*exp(-(2*k-1)^2*pi^2*T/9).*cos((2*k-1)*pi*X/3)/(2*k-1)^2;
end
surf(X,T,uAn);                    % surface plot using surf
xlabel('distance, x'); ylabel('time, t'); zlabel('temperature, u(x,t)');
set(gca,'FontSize',14,'zLim',[1.5 4.5]); box off; view([-33 10]);

% 2D cross-section plots u(x) for fixed t
figure(4); hold on;
lType = {':r',':b','--r','--b','r','b'}; % line types (different for all cases)
x = linspace(0,3,100);                   % discretizing (finer) x-interval
t = [0 0.5 1 2 3 4];                     % discretizing (new) t-interval
for n = 1:length(t)
  uAn = 3*ones(size(x));                 % solution function on x-grid
  for k = 1:50                           % updating solution via truncated summation
    uAn = uAn - (12/pi^2)*exp(-(2*k-1)^2*pi^2*t(n)/9).*cos((2*k-1)*pi*x/3)/(2*k-1)^2;
  end
  plot(x,uAn,lType{n},'LineWidth',2);
end
xlabel('distance, x'); ylabel('temperature, u(x)');
legend('t = 0','t = 0.5','t = 1','t = 2','t = 3','t = 4','Location','NorthWest');
set(gca,'FontSize',14); legend boxoff; box off;
