% Chapter_6_BVP_implicit.m
%
% An Introduction to Partial Differential Equations with MATLAB
% Third Edition
% Chapter 6: Review of Numerical Methods for Solving ODEs
% Section 6.5: Implicit Approximations for BVPs
% Example 1.
%
%  2024 by Matthew P. Coleman and Vladislav Bukshtynov
% VladislavBukshtynov@yahoo.com
%

close all; clear; clc;

% Example 1: solving BVP numerically
% y'' + 4y = 0, y(0) = y(Pi) = 1
x0 = 0; xf = pi; y0 = 1; yf = 1; % boundary conditions
n = [4 10 100];                  % # of inner points

% analytical solution
% y(x) = cos(2x)
yAn=@(x) cos(2*x); % analytical solution through a function handle
xx = 0:0.001:pi;   % fine x-scale discretization
yy = yAn(xx);      % analytical solution evaluated at a fine scale

% numerical solution via implicit (central FD) method
% calling function bvp2implicit(x0,y0,xf,yf,n)
[x1, y1] = bvp2implicit(x0,y0,xf,yf,n(1)); % solution with n = 4
[x2, y2] = bvp2implicit(x0,y0,xf,yf,n(2)); % solution with n = 10
[x3, y3] = bvp2implicit(x0,y0,xf,yf,n(3)); % solution with n = 100

% plotting all solutions
figure(1); hold on;
plot(xx,yy,'-r','LineWidth',2);
plot(x1,y1,'--sb','LineWidth',1,'MarkerSize',12,'MarkerFaceColor','b');
plot(x2,y2,'--^b','LineWidth',1,'MarkerSize',10,'MarkerFaceColor','b');
plot(x3,y3,'--ob','LineWidth',1,'MarkerSize',7);
xlabel('x'); ylabel('y(x)');
legend('y_{ex}(x)','n = 4','n = 10','n = 100','Location','North');
set(gca,'FontSize',14); legend boxoff; box off;

% error analysis
absErr1 = abs(y1-yAn(x1));           % absolute error
absErr2 = abs(y2-yAn(x2));
absErr3 = abs(y3-yAn(x3));
relErr1 = 100*absErr1./abs(yAn(x1)); % relative error
relErr2 = 100*absErr2./abs(yAn(x2));
relErr3 = 100*absErr3./abs(yAn(x3));

% plotting (log-scaled) errors
figure(2); % relative error
semilogy(x1,relErr1,'--sb','LineWidth',1,'MarkerSize',10); hold on;
semilogy(x2,relErr2,'--^b','LineWidth',1,'MarkerSize',10);
semilogy(x3,relErr3,'--ob','LineWidth',1,'MarkerSize',10); hold off;
xlabel('x'); ylabel('relative error (%), \epsilon_r');
legend('n = 4','n = 10','n = 100','Location','SouthEast');
set(gca,'FontSize',14); legend boxoff; box off;

figure(3); % absolute error
semilogy(x1,absErr1,'--sb','LineWidth',1,'MarkerSize',10); hold on;
semilogy(x2,absErr2,'--^b','LineWidth',1,'MarkerSize',10);
semilogy(x3,absErr3,'--ob','LineWidth',1,'MarkerSize',10); hold off;
xlabel('x'); ylabel('absolute error, \epsilon_a');
legend('n = 4','n = 10','n = 100','Location','SouthEast');
set(gca,'FontSize',14); legend boxoff; box off;

figure(4); % convergence by 2-nd norm analysis
convNorm = [norm(absErr1,2) norm(absErr2,2) norm(absErr3,2)];
loglog(n,convNorm,'r.-','LineWidth',1,'MarkerSize',25);
xlabel('n'); ylabel('error norm');
set(gca,'FontSize',14); box off;

return

% function: solving BVP-2 via implicit (central FD) method
function [x, y] = bvp2implicit(x0,y0,xf,yf,n)
  x = linspace(x0,xf,n+2);  % [x0 xf] discretization by n inner points
  h = (xf-x0)/(n+1);        % discretization step
  A = diag((4*h^2-2)*ones(n,1),0) ...
    + diag(ones(n-1,1),1) ...
    + diag(ones(n-1,1),-1); % tridiagonal matrix A
  b = zeros(n,1);           % RHS vector b initialization
  b(1) = b(1) - y0;         % adding left BC
  b(end) = b(end) - yf;     % adding right BC
  y = A^-1*b;               % numerical solution by linear algebra
  y = [y0; y; yf]';         % adding known solution at boundaries
end
