% Chapter_6_ODEs_by_RK_higher_order.m
%
% An Introduction to Partial Differential Equations with MATLAB
% Third Edition
% Chapter 6: Review of Numerical Methods for Solving ODEs
% Section 6.4: Solving Higher-Order ODEs Numerically
% Example 1.
%
%  2024 by Matthew P. Coleman and Vladislav Bukshtynov
% VladislavBukshtynov@yahoo.com
%

close all; clear; clc;

% Example 1: solving IVP numerically
% y'' + 4y = 0, y(0) = 1, y'(0) = 0, find y(Pi)
fRHS=@(x,y,u) -4*y;     % du/dx = -4y, where u = y'
x0 = 0; y0 = 1; u0 = 0; % initial conditions
xt = pi;                % terminal value of x

% analytical solution
% y(x) = cos(2x)
yAn=@(x) cos(2*x);  % analytical solution through a function handle
xf = x0:0.01:xt;    % fine x-scale discretization
yf = yAn(xf);       % analytical solution evaluated at a fine scale

% numerical solutions via RK methods
xx = x0:pi/19:xt;            % discretizing [x0, xt] with step h = pi/19
y1 = rk1ODE2(xx,y0,u0,fRHS); % solution by RK-1
y4 = rk4ODE2(xx,y0,u0,fRHS); % solution by RK-4

% plotting all solutions
figure(1); hold on;
plot(xf,yf,'-r','LineWidth',2);
plot(xx,y1,'--sb','LineWidth',1,'MarkerSize',10);
plot(xx,y4,'--ob','LineWidth',1,'MarkerSize',10);
xlabel('x'); ylabel('y(x)');
legend('y_{ex}(x)','RK-1','RK-4','Location','North');
set(gca,'FontSize',14); legend boxoff; box off;

% numerical solutions using built-in functions
[xN23, yN23] = ode23(@odes,xx,[u0 y0]); % solution by ode23
[xN45, yN45] = ode45(@odes,xx,[u0 y0]); % solution by ode45

% plotting all solutions
figure(2); hold on;
plot(xf,yf,'-r','LineWidth',2);
plot(xN23,yN23(:,2),'--sk','LineWidth',1,'MarkerSize',10);
plot(xN45,yN45(:,2),'--ok','LineWidth',1,'MarkerSize',10);
xlabel('x'); ylabel('y(x)');
legend('y_{ex}(x)','ode23','ode45','Location','North');
set(gca,'FontSize',14); legend boxoff; box off;

% error analysis
absErr1 = abs(y1-yAn(xx));           % for RK-1
absErr4 = abs(y4-yAn(xx));           % for RK-4
absErr23 = abs(yN23(:,2)-yAn(xN23)); % for ode23
absErr45 = abs(yN45(:,2)-yAn(xN45)); % for ode45

% plotting errors
figure(3);
semilogy(xx(2:end),absErr1(2:end),'--sb','LineWidth',1,'MarkerSize',10); hold on;
semilogy(xx(2:end),absErr4(2:end),'--ob','LineWidth',1,'MarkerSize',10);
semilogy(xN23(2:end),absErr23(2:end),'--sk','LineWidth',1,'MarkerSize',10);
semilogy(xN45(2:end),absErr45(2:end),'--ok','LineWidth',1,'MarkerSize',10); hold off;
xlabel('x'); ylabel('absolute error, \epsilon_a');
legend('RK-1','RK-4','ode23','ode45','Location','SouthEast');
set(gca,'FontSize',14); legend boxoff; box off;

return

% function: RK-1 implementation for ODE-2
function y = rk1ODE2(x,y0,u0,f)
  y = zeros(size(x));
  u = zeros(size(x));
  y(1) = y0;
  u(1) = u0;
  for i = 2:length(x)
    h = x(i) - x(i-1);
    y(i) = y(i-1) + h*u(i-1);
    u(i) = u(i-1) + h*f(x(i-1),y(i-1),u(i-1));
  end
end

% function: RK-4 implementation for ODE-2
function y = rk4ODE2(x,y0,u0,f)
  y = zeros(size(x));
  u = zeros(size(x));
  y(1) = y0;
  u(1) = u0;
  for i = 2:length(x)
    h = x(i) - x(i-1);
    m1 = u(i-1);
    k1 = f(x(i-1),y(i-1),u(i-1));
    m2 = u(i-1) + h*k1/2;
    k2 = f(x(i-1)+h/2,y(i-1)+h*m1/2,u(i-1)+h*k1/2);
    m3 = u(i-1) + h*k2/2;
    k3 = f(x(i-1)+h/2,y(i-1)+h*m2/2,u(i-1)+h*k2/2);
    m4 = u(i-1) + h*k3;
    k4 = f(x(i-1)+h,y(i-1)+h*m3,u(i-1)+h*k3);
    y(i) = y(i-1) + h*(m1+2*m2+2*m3+m4)/6;
    u(i) = u(i-1) + h*(k1+2*k2+2*k3+k4)/6;
  end
end

% ODE-2 as a system of two ODE-1 for ode23 and ode45
function dydx = odes(x,y)
  dydx = [y(2)     % u = y'
         -4*y(1)]; % u' = y'' = -4y
end
