% Chapter_6_ODEs_by_RK_methods.m
%
% An Introduction to Partial Differential Equations with MATLAB
% Third Edition
% Chapter 6: Review of Numerical Methods for Solving ODEs
% Section 6.3: Numerical Solutions by Runge-Kutta Methods
% Examples 1-3.
%
%  2024 by Matthew P. Coleman and Vladislav Bukshtynov
% VladislavBukshtynov@yahoo.com
%

close all; clear; clc;

% Example 1: solving IVP numerically
% y' = x*y^2 - y/x, y(1) = 1
fRHS=@(x,y) x.*y.^2 - y./x;  % RHS function through a function handle
x0 = 1; y0 = 1;              % initial conditions
xf = 1.5;                    % terminal value of x

% analytical solution (Bernoulli's equation)
% y(x) = 1/(2x - x^2)
yAn=@(x) 1./(2*x - x.^2);  % analytical solution through a function handle
xx = x0:0.01:xf;           % fine x-scale discretization
yy = yAn(xx);              % analytical solution evaluated at a fine scale

% numerical solutions via RK methods
x1 = x0:0.25:xf;       % discretizing [x0, xf] with step h = 0.25
y1 = rk1(x1,y0,fRHS);  % solution by RK-1
y2 = rk2(x1,y0,fRHS);  % solution by RK-2
y4 = rk4(x1,y0,fRHS);  % solution by RK-4

% error assesment
disp('Example 1:');
disp('Error assessment (RK-1 vs. RK-2 vs. RK-4), h = 0.25:')
fprintf(' xn     RK-1     RK-2     RK-4    act_value         abs_error              rel_error\n');
for i = 1:length(x1)
  abs_err_1 = abs(yAn(x1(i))-y1(i));
  abs_err_2 = abs(yAn(x1(i))-y2(i));
  abs_err_4 = abs(yAn(x1(i))-y4(i));
  rel_err_1 = abs_err_1*100/abs(yAn(x1(i)));
  rel_err_2 = abs_err_2*100/abs(yAn(x1(i)));
  rel_err_4 = abs_err_4*100/abs(yAn(x1(i)));
  fprintf('%.2f   %.4f   %.4f   %.4f    %.4f     %.4f|%.4f|%.4f     %.2f%%|%.2f%%|%.2f%%\n',...
      x1(i),y1(i),y2(i),y4(i),yAn(x1(i)),abs_err_1,abs_err_2,abs_err_4,rel_err_1,rel_err_2,rel_err_4);
end

% plotting all solutions
figure(1); hold on;
plot(xx,yy,'-r','LineWidth',2);
plot(x1,y1,'--sb','LineWidth',1,'MarkerSize',10);
plot(x1,y2,'--^b','LineWidth',1,'MarkerSize',10);
plot(x1,y4,'--ob','LineWidth',1,'MarkerSize',10);
axis([0.9 1.6 0.95 1.35]); xlabel('x'); ylabel('y');
legend('y_{ex}(x)','RK-1 (Euler)','RK-2 (improved Euler)','RK-4','Location','NorthWest');
set(gca,'FontSize',14); legend boxoff; box off;
pause;             % pausing output - to proceed push any key

% Example 2: solving IVP numerically
% y' = 2y cos(x), y(0) = 1, find y(9)
fRHS=@(x,y) 2*y.*cos(x);     % RHS function through a function handle
x0 = 0; y0 = 1;              % initial conditions
xf = 9;                      % terminal value of x

% analytical solution (separable equation)
% y(x) = exp(2*sin(x))
yAn=@(x) exp(2*sin(x));    % analytical solution through a function handle
xx = x0:0.01:xf;           % fine x-scale discretization
yy = yAn(xx);              % analytical solution evaluated at a fine scale

% numerical solution via RK methods
x1 = x0:1.0:xf;       % discretizing [x0, xf] with step h = 1.0
y1 = rk1(x1,y0,fRHS); % solution by RK-1
y2 = rk2(x1,y0,fRHS); % solution by RK-2
y4 = rk4(x1,y0,fRHS); % solution by RK-4

% plotting all solutions
figure(2); hold on;
plot(xx,yy,'-r','LineWidth',2);
plot(x1,y1,'--sb','LineWidth',1,'MarkerSize',10);
plot(x1,y2,'--^b','LineWidth',1,'MarkerSize',10);
plot(x1,y4,'--ob','LineWidth',1,'MarkerSize',10);
axis([0 10 -2 9]); xlabel('x'); ylabel('y');
legend('y_{ex}(x)','RK-1 (Euler)','RK-2 (improved Euler)','RK-4','Location','North');
set(gca,'FontSize',14); legend boxoff; box off;
pause;             % pausing output - to proceed push any key

% Example 3: example 2 modified
% y' = 2y cos(x), y(0) = 1, find y(30)
xf = 30;               % terminal value of x

% numerical solution via RK methods
xa = x0:1.0:xf;        % discretizing [x0, xf] with step h = 1.0
y1a = rk1(xa,y0,fRHS); % solution by RK-1
y4a = rk4(xa,y0,fRHS); % solution by RK-4
xb = x0:0.99:xf;       % discretizing [x0, xf] with step h = 0.99
y1b = rk1(xb,y0,fRHS); % solution by RK-1
y4b = rk4(xb,y0,fRHS); % solution by RK-4

% analytical solution (for modified interval)
xx = x0:0.01:xf;       % fine x-scale discretization
yy = yAn(xx);          % analytical solution evaluated at a fine scale

% plotting all solutions
figure(3); hold on;
plot(xx,yy,'-r','LineWidth',2);
plot(xa,y1a,'--sb','LineWidth',1,'MarkerSize',10);
plot(xa,y1b,'--sb','LineWidth',1,'MarkerSize',7,'MarkerFaceColor','b');
axis([0 30 -10 40]); xlabel('x'); ylabel('y');
legend('y_{ex}(x)','RK-1: h = 1.0','RK-1: h = 0.99','Location','NorthWest');
set(gca,'FontSize',14); legend boxoff; box off;
figure(4); hold on;
plot(xx,yy,'-r','LineWidth',2);
plot(xa,y4a,'--ob','LineWidth',1,'MarkerSize',10);
plot(xa,y4b,'--ob','LineWidth',1,'MarkerSize',7,'MarkerFaceColor','b');
axis([0 30 -2 23]); xlabel('x'); ylabel('y');
legend('y_{ex}(x)','RK-4: h = 1.0','RK-4: h = 0.99','Location','NorthWest');
set(gca,'FontSize',14); legend boxoff; box off;

return

% function: RK-1 (Euler's) method implementation
function y = rk1(x,y0,f)
  y = size(x);
  y(1) = y0;
  for i = 2:length(x)
    h = x(i) - x(i-1);
    y(i) = y(i-1) + h*f(x(i-1),y(i-1));
  end
end

% function: RK-2 (improved Euler's) method implementation
function y = rk2(x,y0,f)
  y = size(x);
  y(1) = y0;
  for i = 2:length(x)
    h = x(i) - x(i-1);
    yPred = y(i-1) + h*f(x(i-1),y(i-1));
    y(i) = y(i-1) + h*(f(x(i-1),y(i-1))+f(x(i),yPred))/2;
  end
end

% function: RK-4 method implementation
function y = rk4(x,y0,f)
  y = size(x);
  y(1) = y0;
  for i = 2:length(x)
    h = x(i) - x(i-1);
    k1 = f(x(i-1),y(i-1));
    k2 = f(x(i-1)+h/2,y(i-1)+h*k1/2);
    k3 = f(x(i-1)+h/2,y(i-1)+h*k2/2);
    k4 = f(x(i-1)+h,y(i-1)+h*k3);
    y(i) = y(i-1) + h*(k1/6 + k2/3 + k3/3 + k4/6);
  end
end
