% Chapter_6_approximate_IVPs.m
%
% An Introduction to Partial Differential Equations with MATLAB
% Third Edition
% Chapter 6: Review of Numerical Methods for Solving ODEs
% Section 6.2: Numerical Solutions Using Euler's Method
% Examples 1 and 2.
%
%  2024 by Matthew P. Coleman and Vladislav Bukshtynov
% VladislavBukshtynov@yahoo.com
%

close all; clear; clc;

% Example 1: solving IVP numerically
% y' = x*y^2 - y/x, y(1) = 1
x0 = 1; y0 = 1;          % initial conditions
xf = 1.5;                % terminal value of x
step = [0.25 0.1 0.025]; % choice of step size
figure(1); hold on;
xx = x0:0.01:xf;         % plotting exact solution
yy = 1./(2*xx - xx.^2);
plot(xx,yy,'-r','LineWidth',2);
lType = {'--sb','--^b','--ob'}; % line types (different for all approximations)
for k = 1:length(step)   % loop over different step sizes h
  h = step(k);
  x(1) = x0; y(1) = y0;
  for i = 1:(xf-x0)/h    % loop over discretized values of x
    x(i+1) = x(i) + h;
    y(i+1) = h * (x(i)*(y(i))^2 - y(i)/x(i)) + y(i);
  end
  plot(x,y,lType{k},'LineWidth',1,'MarkerSize',10); % plotting solution
  err = abs((y(end)-yy(end))/yy(end))*100;          % computing relative error
  fprintf('h = %1.3f,  y(1.5): %1.4f(ex) %1.4f(approx),  err = %2.2f%%\n',h,yy(end),y(end),err); % output
end
xlabel('x'); ylabel('y'); axis([0.9 1.6 0.95 1.35]);
legend('y_{ex}(x)','h = 0.25','h = 0.1','h = 0.025','Location','NorthWest');
set(gca,'FontSize',14); legend boxoff; box off;
pause;             % pausing output - to proceed push any key

% Example 2: solving IVP numerically
% y' = -2y, y(0) = 1
clear;
x0 = 0; y0 = 1;          % initial conditions
xf = 5;                  % terminal value of x
step = [1.05 0.85 0.25]; % choice of step size
figure(2); hold on;
xx = x0:0.01:xf;         % plotting exact solution
yy = exp(-2*xx);
plot(xx,yy,'-r','LineWidth',2);
lType = {'--sb','--^b','--ob'}; % line types (different for all approximations)
for k = 1:length(step)   % loop over different step sizes h
  h = step(k);
  x(1) = x0; y(1) = y0;
  for i = 1:(xf-x0)/h    % loop over discretized values of x
    x(i+1) = x(i) + h;
    y(i+1) = (1-2*h)^(i+2)*y(1);
  end
  plot(x,y,lType{k},'LineWidth',1,'MarkerSize',10); % plotting solution
end
xlabel('x'); ylabel('y');
legend('y_{ex}(x)','h = 1.05','h = 0.85','h = 0.25','Location','SouthEast');
set(gca,'FontSize',14); legend boxoff; box off;
