% Chapter_7_heat_equation.m
%
% An Introduction to Partial Differential Equations with MATLAB
% Third Edition
% Chapter 7: Solving PDEs by Finite Difference Approximations
% Section 7.1: Numerical Solutions for Heat Equations
% Solving the Heat Equation by two methods:
%   (1) explicit, and
%   (2) Crank-Nicolson scheme
%
%  2024 by Matthew P. Coleman and Vladislav Bukshtynov
% VladislavBukshtynov@yahoo.com
%

close all; clear; clc;
set(groot, 'DefaultFigureColormap', jet);

% Example:
% u_t = a^2 u_xx, 0 < x < 1, t > 0
% IC:  u(x,0) = sin(pi*x)
% BCs: u(0,t) = u(1,t) = 0

% defining constants
a  = 1;
x0 = 0; xf = 1;   % space interval
t0 = 0; tf = 0.2; % time interval
n = 6; m = 11;    % number of points (space & time)

% discretization
dx = (xf-x0)/(n-1); % step size for space
dt = (tf-t0)/(m-1); % step size for time

% initial condition RHS function f(x)=sin(pi*x)
f=@(dx,i) sin(pi*dx*i);

% numerical solutions with both methods
uEx = heatEqn_explicit(a,dx,dt,n,m,f);      % explicit scheme
uCN = heatEqn_CrankNicolson(a,dx,dt,n,m,f); % Crank-Nicolson scheme

% plotting solutions
[x,t] = meshgrid(x0:dx:xf, t0:dt:tf);

% analytical solution
figure(1);
uAn = exp(-pi^2*t).*sin(pi*x);
surf(x,t,uAn);
xlabel('x'); ylabel('t'); zlabel('u(x,t)');
title('Analytical Solution');
set(gca,'FontSize',14); view([-32 12]); box off;

% explicit scheme
figure(2);
surf(x,t,uEx);
xlabel('x'); ylabel('t'); zlabel('u(x,t)');
title('Explicit Numerical Solution');
set(gca,'FontSize',14); view([-32 12]); box off;

% Crank-Nicolson scheme
figure(3);
surf(x,t,uCN);
xlabel('x'); ylabel('t'); zlabel('u(x,t)');
title('Crank-Nicolson Numerical Solution');
set(gca,'FontSize',14); view([-32 12]); box off;

% error analysis for fixed x-values
uEx_relErr = 100*abs(uAn-uEx)./abs(uAn); % explicit
uCN_relErr = 100*abs(uAn-uCN)./abs(uAn); % Crank-Nicolson

% plotting errors
figure(4); hold on;
plot(t(:,1),uEx_relErr(:,2),'.b','MarkerSize',10);
plot(t(:,1),uCN_relErr(:,2),'.r','MarkerSize',10);
plot(t(:,1),uEx_relErr(:,3),'ob','MarkerSize',8);
plot(t(:,1),uCN_relErr(:,3),'or','MarkerSize',8);
plot(t(:,1),uEx_relErr(:,4),'ob','MarkerSize',12);
plot(t(:,1),uCN_relErr(:,4),'or','MarkerSize',12);
xlabel('t'); ylabel('error, %');
legend('x=0.2 explicit','x=0.2 Crank-Nicolson',...
       'x=0.4 explicit','x=0.4 Crank-Nicolson',...
       'x=0.6 explicit','x=0.6 Crank-Nicolson','Location','NorthWest');
set(gca,'FontSize',14); legend boxoff; box off;

% Same example with unstable explicit scheme
m = 6;              % number of points (time only)

% discretization
dx = (xf-x0)/(n-1); % step size for space
dt = (tf-t0)/(m-1); % step size for time

% numerical solutions with both methods
uEx = heatEqn_explicit(a,dx,dt,n,m,f);      % explicit scheme
uCN = heatEqn_CrankNicolson(a,dx,dt,n,m,f); % Crank-Nicolson scheme

% plotting solutions
[x,t] = meshgrid(x0:dx:xf, t0:dt:tf);

% analytical solution
uAn = exp(-pi^2*t).*sin(pi*x);

% explicit scheme
figure(5);
surf(x,t,uEx);
xlabel('x'); ylabel('t'); zlabel('u(x,t)');
title('Explicit Numerical Solution');
set(gca,'FontSize',14); view([-32 12]); box off;

% Crank-Nicolson scheme
figure(6);
surf(x,t,uCN);
xlabel('x'); ylabel('t'); zlabel('u(x,t)');
title('Crank-Nicolson Numerical Solution');
set(gca,'FontSize',14); view([-32 12]); box off;

% error analysis for fixed x-values
uEx_relErr = 100*abs(uAn-uEx)./abs(uAn); % explicit
uCN_relErr = 100*abs(uAn-uCN)./abs(uAn); % Crank-Nicolson

% plotting errors
figure(7); hold on;
plot(t(:,1),uEx_relErr(:,2),'.b','MarkerSize',10);
plot(t(:,1),uCN_relErr(:,2),'.r','MarkerSize',10);
plot(t(:,1),uEx_relErr(:,3),'ob','MarkerSize',8);
plot(t(:,1),uCN_relErr(:,3),'or','MarkerSize',8);
plot(t(:,1),uEx_relErr(:,4),'ob','MarkerSize',12);
plot(t(:,1),uCN_relErr(:,4),'or','MarkerSize',12);
xlabel('t'); ylabel('error, %');
legend('x=0.2 explicit','x=0.2 Crank-Nicolson',...
       'x=0.4 explicit','x=0.4 Crank-Nicolson',...
       'x=0.6 explicit','x=0.6 Crank-Nicolson','Location','NorthWest');
set(gca,'FontSize',14); legend boxoff; box off;

return

% explicit numerical scheme
function sol = heatEqn_explicit(a,dx,dt,n,m,f)
  eps = (a^2)*dt/(dx^2);
  
  % solution mesh (including homogeneous Dirichlet boundary conditions)
  U = zeros(n,m);
  
  % initial condition
  for i = 1:n
    U(i,1) = f(dx,i-1);
  end  
  
  % marching in time
  for j = 1:m-1
    for i = 2:n-1
      U(i,j+1) = eps*U(i+1,j) + (1-2*eps)*U(i,j) + eps*U(i-1,j);
    end
  end
  sol = U'; % complete solution
end

% Crank-Nicolson scheme
function sol = heatEqn_CrankNicolson(a,dx,dt,n,m,f)
  eps = (a^2)*dt/(dx^2);
  
  % solution mesh (including homogeneous Dirichlet boundary conditions)
  U = zeros(n,m);
  
  % initial condition
  for i = 1:n
    U(i,1) = f(dx,i-1);
  end
  
  % tridiagonal matrix to be used for implicit marching in time
  A = diag(2*(1+eps)*ones(n-2,1),0) + diag(-eps*ones(n-3,1),1) + diag(-eps*ones(n-3,1),-1);

  % marching in time
  for j = 1:m-1
    for i = 1:n-2 % creating RHS vector B
      B(i) = eps*U(i,j) + 2*(1-eps)*U(i+1,j) + eps*U(i+2,j);
    end
    B(1) = B(1) + eps*U(1,j+1);       % adding left BC
    B(end) = B(end) + eps*U(end,j+1); % adding right BC
    U(2:end-1,j+1) = A^-1*B';  
  end
  sol = U'; % complete solution
end
