% Chapter_7_laplace_equation.m
%
% An Introduction to Partial Differential Equations with MATLAB
% Third Edition
% Chapter 7: Solving PDEs by Finite Difference Approximations
% Section 7.3: Numerical Schemes for Laplace's Equations
% Solving Laplace's Equation by Jacobi iteration scheme
%
%  2024 by Matthew P. Coleman and Vladislav Bukshtynov
% VladislavBukshtynov@yahoo.com
%

close all; clear; clc;
set(groot, 'DefaultFigureColormap', jet);

% Example:
% u_xx + u_yy = 0, 0 < x < 1, 0 < y < 1
% u(0,y) = u(1,y) = 0
% u(x,0) = sin(2pi*x); u(x,1) = sin(pi*x)

% defining constants
a = 1; b = 1;   % size of (x,y)-domain
n = 11; m = 11; % number of points (in x- & y-space)

% discretization
dx = a/(n-1);   % step size for x
dy = b/(m-1);   % step size for y
[x,y] = meshgrid(0:dx:a,0:dy:b);
U = zeros(n,m); % solution mesh incl. homogeneous Dirichlet BCs for x = 0 & x = 1

% analytical solution
uAn = (sin(pi*x).*sinh(pi*y)./sinh(pi)) + ...
      (sin(2*pi*x).*(cosh(2*pi*y) - ...
      (cosh(2*pi)./sinh(2*pi)).*sinh(2*pi*y)));
figure(1); subplot(1,2,1); surf(x,y,uAn);
xlabel('x'); ylabel('y'); zlabel('u(x,y)'); title('Analytical Solution');
set(gca,'FontSize',14); view([-28 18]); box off;

% adding rest boundary conditions: u(x,0) = sin(2pi*x), u(x,1) = sin(pi*x)
for i = 1:n
  U(i,1) = sin(2*pi*dx*(i-1));
  U(i,m) = sin(pi*dx*(i-1));
end

% initial guess (if different from 0)
for i = 2:n-1
  for j = 2:m-1
    % put here any idea on guess, otherwise it is 0
  end
end
U = U';

% convergence analysis initialization
errConv = [norm(uAn-U,inf) norm(uAn-U,2) 0 0];

% plotting initial solution (guess)
figure(1); subplot(1,2,2); surf(x,y,U);
title('Jacobi Iteration, N = 0');
xlabel('x'); ylabel('y'); zlabel('u(x,y)');
set(gca,'FontSize',14); view([-28 18]); box off; pause(1);

% Jacobi iteration
for k = 1:100  % loop over 100 iterations
  Up = U;    % saving solution from previous iteration
  for j = 2:n-1
    for i = 2:m-1
      U(i,j)=0.25*(Up(i,j+1)+Up(i,j-1)+Up(i+1,j)+Up(i-1,j));
    end
  end
  figure(1); subplot(1,2,2); surf(x,y,U); % plotting current solution
  title(['Jacobi Iteration, N = ', num2str(k)]);
  xlabel('x'); ylabel('y'); zlabel('u(x,y)');
  set(gca,'FontSize',14); view([-28 18]); box off;
  errConv = [errConv; norm(uAn-U,inf) norm(uAn-U,2) norm(U-Up,inf) norm(U-Up,2)];
  pause(0.1);
end

% computing absolute error
absErr = abs(uAn-U);

% plotting errors for fixed x-values
figure(2); hold on;
plot(y(:,1),absErr(:,2),'b-o','LineWidth',1);
plot(y(:,1),absErr(:,4),'r-o','LineWidth',1);
plot(y(:,1),absErr(:,6),'b--o','LineWidth',1);
plot(y(:,1),absErr(:,8),'r--o','LineWidth',1);
xlabel('y'); ylabel('error'); title('Absolute error at N = 100 for fixed x');
legend('x = 0.1','x = 0.3','x = 0.5','x = 0.7','Location','NorthEast');
set(gca,'FontSize',14); legend boxoff; box off;

% plot errors for fixed y-values
figure(3); hold on;
plot(x(1,:),absErr(3,:),'b-o','LineWidth',1);
plot(x(1,:),absErr(5,:),'r-o','LineWidth',1);
plot(x(1,:),absErr(7,:),'b--o','LineWidth',1);
plot(x(1,:),absErr(9,:),'r--o','LineWidth',1);
xlabel('x'); ylabel('error'); title('Absolute error at N = 100 for fixed y');
legend('y = 0.2','y = 0.4','y = 0.6','y = 0.8','Location','North');
set(gca,'FontSize',14); legend boxoff; box off;

% convergence analysis: comparison with analytical solution
figure(4);
semilogy(0:k,errConv(:,1),'bo-','LineWidth',1); hold on;
semilogy(0:k,errConv(:,2),'ro-','LineWidth',1);
xlabel('iteration, N'); ylabel('error');
title('Convergence analysis using analytical solution');
legend('\infty-norm','2-norm');
set(gca,'FontSize',14); legend boxoff; box off;

% convergence analysis: comparison with solution at previous iteration
figure(5);
semilogy(0:k,errConv(:,3),'bo-','LineWidth',1); hold on;
semilogy(0:k,errConv(:,4),'ro-','LineWidth',1);
xlabel('iteration, N'); ylabel('error');
title('Convergence Analysis using Previous Solution');
legend('\infty-norm','2-norm');
set(gca,'FontSize',14); legend boxoff; box off;
