% Chapter_7_wave_equation.m
%
% An Introduction to Partial Differential Equations with MATLAB
% Third Edition
% Chapter 7: Solving PDEs by Finite Difference Approximations
% Section 7.2: Explicit Scheme for Wave Equations
% Solving the Wave Equation by the explicit scheme
%
%  2024 by Matthew P. Coleman and Vladislav Bukshtynov
% VladislavBukshtynov@yahoo.com
%

close all; clear; clc;
set(groot, 'DefaultFigureColormap', jet);

% Example:
% u_tt = c^2 u_xx, 0 < x < 1, t > 0
% ICs: u(x,0) = sin(pi*x),  u_t(x,0) = 0
% BCs: u(0,t) = u(1,t) = 0

% defining constants
c  = 1;
x0 = 0; xf = 1;   % space interval
t0 = 0; tf = 2;   % time interval
n = 6; m = 21;    % number of points (space & time)

% discretization
dx = (xf-x0)/(n-1); % step size for space
dt = (tf-t0)/(m-1); % step size for time

% initial condition functions
f=@(dx,i) sin(pi*dx*i); % u(x,0) = sin(pi*x)
g=@(dx,i) 0;            % u_t(x,0) = 0

% numerical solution (for dt = 0.1)
U1 = waveEqn(c,dx,dt,n,m,f,g); % calling user function waveEqn

% plotting solutions
[x,t1] = meshgrid(x0:dx:xf, t0:dt:tf);

% analytical solution
figure(1);
uAn1 = cos(pi*t1).*sin(pi*x);
surf(x,t1,uAn1);
xlabel('x'); ylabel('t'); zlabel('u(x,t)');
title('Analytical Solution');
set(gca,'FontSize',14); view([-40 10]); box off;

% numerical (explicit) scheme
figure(2);
surf(x,t1,U1);
xlabel('x'); ylabel('t'); zlabel('u(x,t)');
title('Numerical Solution by Explicit Scheme, \Delta t = 0.1');
set(gca,'FontSize',14); view([-40 10]); box off;

% numerical solution (for dt = 0.5)
m = 5;                         % number of points (time)
dt = (tf-t0)/(m-1);            % step size for time
[x,t2] = meshgrid(x0:dx:xf, t0:dt:tf);
uAn2 = cos(pi*t2).*sin(pi*x);  % analytical solution on new mesh
U2 = waveEqn(c,dx,dt,n,m,f,g); % calling user function waveEqn

% plotting solutions
figure(3);
surf(x,t2,U2);
xlabel('x'); ylabel('t'); zlabel('u(x,t)');
title('Numerical Solution by Explicit Scheme, \Delta t = 0.5');
set(gca,'FontSize',14); view([-40 10]); box off;

% error analysis at x = 0.4
absErr1 = abs(uAn1-U1); relErr1 = 100*absErr1./abs(uAn1);
absErr2 = abs(uAn2-U2); relErr2 = 100*absErr2./abs(uAn2);

% plotting relative errors
figure(4);
semilogy(t1(:,1),relErr1(:,2),'r-o','lineWidth',1); hold on;
semilogy(t2(:,1),relErr2(:,2),'b-o','LineWidth',1);
xlabel('t'); ylabel('relative error (%), \epsilon_r'); title('Relative Error at x = 0.4');
legend('\Delta t = 0.1','\Delta t = 0.5','Location','North');
set(gca,'FontSize',14); legend boxoff; box off;

% plotting absolute errors
figure(5); hold on;
plot(t1(:,1),absErr1(:,2),'r-o','LineWidth',1); hold on;
plot(t2(:,1),absErr2(:,2),'b-o','LineWidth',1);
xlabel('t'); ylabel('absolute error, \epsilon_a'); title('Absolute Error at x = 0.4');
legend('\Delta t = 0.1','\Delta t = 0.5','Location','North');
set(gca,'FontSize',14); legend boxoff; box off;

return

% explicit numerical scheme
function sol = waveEqn(c,dx,dt,n,m,f,g)
  eps = (c*dt/dx)^2;
  
  % solution mesh (including homogeneous Dirichlet boundary conditions)
  U = zeros(n,m+1); % 'j=-1' level added for ghost points
  
  % initial conditions
  for i = 1:n
    U(i,2) = f(dx,i-1);           % u(x,0) = f(x)
    U(i,1) = U(i,2)-g(dx,i-1)*dt; % u(x,-dt) = u(x,0)-g*dt (ghosts)
  end  
  
  % marching in time
  for j = 2:m
    for i = 2:n-1
      U(i,j+1) = eps*U(i+1,j) + 2*(1-eps)*U(i,j) + eps*U(i-1,j) - U(i,j-1);
    end
  end
  sol = U(:,2:end)'; % complete solution
end
