% Chapter_8_integral_transforms_graphs.m
%
% An Introduction to Partial Differential Equations with MATLAB
% Third Edition
% Chapter 8: Integral Transforms
% Section 8.1: The Laplace Transform for PDEs
% Section 8.3: The Fourier Transform
% Section 8.4: The Infinite and Semi-Infinite Heat Equations
%
%  2024 by Matthew P. Coleman and Vladislav Bukshtynov
% VladislavBukshtynov@yahoo.com
%

close all; clear; clc;

% Plotting Figure 8.1 in Example 1 for Section 8.1
x = 0:0.01:2;                    % defining spacial discretization (mesh) and functions
y = erf(x);
z = erfc(x);
figure(1); hold on;
plot(x,y,'-r','LineWidth',2.5);  % plotting error function
plot(x,z,'-b','LineWidth',2.5);  % plotting complementary error function
legend('error function','complementary error function','Location','Best');
legend boxoff; box off; set(gca,'FontSize',14);

% Plotting Figure 8.2(a) in Example 3 for Section 8.3
x = -10:0.01:10;                 % defining spacial discretization (mesh) and functions
f = exp(-4*x.^2);
F = 1/(2*sqrt(2))*exp(-x.^2/16);
figure(2); hold on;
plot(x,f,'-r','LineWidth',2.5);  % plotting Gaussian
plot(x,F,'-b','LineWidth',2.5);  % plotting its Fourier transform
leg = legend('$f(x) = e^{-4x^2}$','$F(\alpha) = \frac{1}{2\sqrt{2}}e^{-\alpha^2/16}$');
set(leg,'Interpreter','latex','Location','NorthWest');
legend boxoff; box off; set(gca,'FontSize',14);

% Plotting Figure 8.2(b) in Example 3 for Section 8.3
x = -10:0.01:10;                 % defining spacial discretization (mesh) and functions
f = exp(-x.^2/16);
F = (2*sqrt(2))*exp(-4*x.^2);
figure(3); hold on;
plot(x,f,'-r','LineWidth',2.5);  % plotting Gaussian
plot(x,F,'-b','LineWidth',2.5);  % plotting its Fourier transform
leg = legend('$f(x) = e^{-x^2/16}$','$F(\alpha) = 2\sqrt{2}e^{-4\alpha^2}$');
set(leg,'Interpreter','latex','Location','NorthWest');
legend boxoff; box off; set(gca,'FontSize',14);

% Plotting Figure 8.3(a) in Example 1 for Section 8.4
xx = -10:0.01:10;                 % defining discretization (space & time) and functions
tt = [1 3 5];
ff = @(z,x,t) exp(-(x-z).^2/(4*t));
u = zeros(3,length(xx));          % u(x,t) for t = 1, 3, and 5
a = 1;
for j = 1:length(xx)
  for i = 1:length(tt)
    u(i,j) = 1/(4*a*sqrt(pi*tt(i))) * integral(@(z) ff(z,xx(j),tt(i)),-a,a);
  end
end
figure(4); hold on;
plot(xx,u(1,:),'-r','LineWidth',2.5);  % plotting solution for t = 1
plot(xx,u(2,:),'-b','LineWidth',2.5);  % plotting solution for t = 3
plot(xx,u(3,:),'--b','LineWidth',2.5); % plotting solution for t = 5
legend('t = 1','t = 3','t = 5','Location','NorthWest');
legend boxoff; box off; set(gca,'FontSize',14);

% Plotting Figure 8.3(b) in Example 1 for Section 8.4
u = zeros(3,length(xx));          % u(x,t) for a = 1, 3, and 5
t = 1;
a = [1 3 5];
for j = 1:length(xx)
  for i = 1:length(a)
    u(i,j) = 1/(4*a(i)*sqrt(pi*t)) * integral(@(z) ff(z,xx(j),t),-a(i),a(i));
  end
end
figure(5); hold on;
plot(xx,u(1,:),'-r','LineWidth',2.5);  % plotting solution for a = 1
plot(xx,u(2,:),'-b','LineWidth',2.5);  % plotting solution for a = 3
plot(xx,u(3,:),'--b','LineWidth',2.5); % plotting solution for a = 5
legend('a = 1','a = 3','a = 5','Location','NorthWest');
legend boxoff; box off; set(gca,'FontSize',14);
