% Chapter_9_pde_by_fourier.m
%
% An Introduction to Partial Differential Equations with MATLAB
% Third Edition
% Chapter 9: Using MATLAB's Symbolic Math Toolbox with Integral Transforms
% Section 9.4: Symbolic Solution of PDEs Using the Fourier Transform
%
%  2024 by Matthew P. Coleman and Vladislav Bukshtynov
% VladislavBukshtynov@yahoo.com
%

close all; clear; clc;
set(groot, 'DefaultFigureColormap', jet);

% Example 1: infinite heat equation
% u_t = u_xx,  -inf < x < inf,  t > 0
% u(x,0) = exp(-x^2)
% lim(|x|->inf) u(x,t) = 0

% Solving PDE by the assisted Fourier transform
sympref('FourierParameters',[1/sqrt(2*pi),-1]); % transform parameters
syms x t w;                      % defining symbolic variables
syms u(x,t) U(t) f(x) F(w);      % defining symbolic functions
f(x) = exp(-x^2);                % initial condition
F(w) = fourier(f)                % applying Fourier transform
ode = diff(U,t) == ((1i*w)^2)*U; % defining ODE
cond = U(0) == F(w);             % IC for ODE
sol(w,t) = dsolve(ode,cond);     % solving ODE
sol = simplify(sol)              % simplifying solution
u(x,t) = ifourier(sol);          % applying inverse transform
u = simplify(u)                  % simplifying solution

% plotting solutions
figure(1);                       % plotting U(w,t)
fsurf(sol,[-3 3 0 1]);
xlabel('\alpha'); ylabel('t'); zlabel('U(\alpha,t)');
set(gca,'FontSize',14); view([-40 20]); box off;
figure(2);                       % plotting u(x,t)
dx = 0.2; dt = 0.1;
[xx,tt] = meshgrid(-1:dx:1,0:dt:1);
uu = eval(u(xx,tt));
surf(xx,tt,uu);
xlabel('x'); ylabel('t'); zlabel('u(x,t)');
set(gca,'FontSize',14); view([-40 20]); box off;

% numerical solution by Crank-Nicolson: -1 < x < 1, 0 < t < 1
n = size(xx,2); m = size(tt,2);
f=@(dx,i) exp(-(-1+dx*i)^2);
uNum = heatEqn_CrankNicolson(1,dx,dt,n,m,f);
figure(3);
surf(xx,tt,uNum);
xlabel('x'); ylabel('t'); zlabel('u(x,t)');
set(gca,'FontSize',14); view([-40 20]); box off;

% error analysis
relErr = 100*abs(uu-uNum)./abs(uu);
figure(4); hold on;
plot(tt(:,1),relErr(:,6),'ro-','LineWidth',1);
plot(tt(:,1),relErr(:,8),'bo-','LineWidth',1);
plot(tt(:,1),relErr(:,10),'ko-','LineWidth',1);
xlabel('t'); ylabel('error, %');
legend('x = 0','x = 0.4','x = 0.8','Location','SouthEast');
set(gca,'FontSize',14); legend boxoff; box off;

return

% Crank-Nicolson scheme
function sol = heatEqn_CrankNicolson(a,dx,dt,n,m,f)
  eps = (a^2)*dt/(dx^2);
  
  % solution mesh (including homogeneous Dirichlet boundary conditions)
  U = zeros(n,m);
  
  % initial condition
  for i = 2:n-1
    U(i,1) = f(dx,i-1);
  end
  
  % tridiagonal matrix to be used for implicit marching in time
  A = diag(2*(1+eps)*ones(n-2,1),0) + diag(-eps*ones(n-3,1),1) + diag(-eps*ones(n-3,1),-1);

  % marching in time
  for j = 1:m-1
    for i = 1:n-2 % creating RHS vector B
      B(i) = eps*U(i,j) + 2*(1-eps)*U(i+1,j) + eps*U(i+2,j);
    end
    B(1) = B(1) + eps*U(1,j+1);       % adding left BC
    B(end) = B(end) + eps*U(end,j+1); % adding right BC
    U(2:end-1,j+1) = A^-1*B';  
  end
  sol = U'; % complete solution
end