% Chapter_9_pde_by_laplace.m
%
% An Introduction to Partial Differential Equations with MATLAB
% Third Edition
% Chapter 9: Using MATLAB's Symbolic Math Toolbox with Integral Transforms
% Section 9.3: Symbolic Solution of PDEs Using the Laplace Transform
%
%  2024 by Matthew P. Coleman and Vladislav Bukshtynov
% VladislavBukshtynov@yahoo.com
%

close all; clear; clc;
set(groot, 'DefaultFigureColormap', jet);

% Example 1: semi-infinite wave equation
% u_tt = u_xx,  x > 0, t > 0
% u(x,0) = u_t(x,0) = 0
% u(0,t) = sin(t)/t
% lim(x->inf) u(x,t) = 0

% Solving PDE by the assisted Laplace transform
syms x t s a;                    % defining symbolic variables
syms u(x,t) U(x) f(t) F(s);      % defining symbolic functions
f(t) = sin(t)/t;                 % boundary condition
F(s) = laplace(f)                % applying Laplace transform
U_x = diff(U);                  
ode = diff(U_x) == (s^2)*U;      % defining ODE
cond = [U(0) == F(s) U(a) == 0]; % BCs for ODE
sol(x,s) = dsolve(ode,cond);     % solving ODE
sol = simplify(sol)              % simplifying solution
sol = subs(sol,a,inf)            % substituting inf-BC
u(x,t) = ilaplace(sol)           % applying inverse transform
syms uAn(x,t);                   % final solution (manually)
uAn(x,t) = piecewise(t > -x, (sin(t+x))/(t+x), t < -x, 0);

% plotting solutions as surfaces
figure(1);                       % plotting U(x,s)
fsurf(sol,[0 10 -3 0]);
xlabel('x'); ylabel('s'); zlabel('U(x,s)');
set(gca,'FontSize',14); view([48 24]); box off;
figure(2);                       % plotting u(x,t)
[xx,tt] = meshgrid(-1:0.1:10,0:0.05:1);
surf(xx,tt,eval(uAn(xx,tt)));
xlabel('x'); ylabel('t'); zlabel('u(x,t)');
set(gca,'FontSize',14); view([15 20]); box off;

% plotting u(x,t) for fixed time t
figure(3);
x = 0:0.1:100;                   % x-space discretization
tf = 10;                         % final time
u = zeros(size(x));              % initial solution u(x,0)
for t = 0:0.1:tf                 % computing and plotting solution u(x,t)
  if t > 0
    u = sin(t+x)./(t+x).*heaviside(t+x);
  end
  plot([x(1) x(end)],[0 0],'k--'); hold on;
  plot(x,u,'b.','MarkerSize',10); hold off; title(['t = ' num2str(t)]);
  axis([x(1) x(end) -0.3 1]); xlabel('x'); ylabel('u(x,t)');
  set(gca,'FontSize',14); box off;
  pause(0.1);
end
